/* 
* TrebEdit
* Copyright (c) 2023 Joel Etetafia
*
* You are not permitted to copy, modify, or use any part of this document.
*
*/
const Handle = {
	Bottom: 0,
	Right: 1,
	Corner: 2
}

/**
 * Defines the different 'width-break-points' for the emulator handles
 */
const WidthPoints = {
    NOT_SET: -1,
    _350_AND_LESS: 0,
    _350_TO_600: 1,
    _601_TO_950: 2,
    _951_TO_1150: 3,
    _1151_AND_MORE: 4
}

const CutoutOrientation = {
    HORIZONTALLY_CENTERED : 0,
    VERTICALLY_CENTERED : 1
}

var iFrame = document.getElementById("iframe");
var iFrameContainer = document.getElementById("iframe_container");
var rightHandle = document.getElementById("right_handle");
var bottomHandle = document.getElementById("bottom_handle");
var cornerHandle = document.getElementById("corner_handle");
var emulatorCutout = document.getElementById("foldable_device_cutout");

const MIN_FRAME_WIDTH = 100;
const MIN_FRAME_HEIGHT = 100;
const IFRAME_SCROLLBAR_WIDTH = 12;

var isResizeEnabled = true;

var currentWidthPoint = WidthPoints.NOT_SET


document.addEventListener('DOMContentLoaded', function (event) {
    if (isResizeObserverSupported()) {
        try {
            var resizer = new ResizeObserver(onResize);
            resizer.observe(iFrame);
        } catch (error) {
            //ResizerObserver is not supported, initialize mutation observer
            initMutationObserver();
        }
    } else {
        //ResizerObserver is not supported, initialize mutation observer
        initMutationObserver();
    }
});

function isResizeObserverSupported() {
    return typeof ResizeObserver != "undefined";
}

var startWidth, startHeight;

//right handle
var rightHandleHammer = new Hammer.Manager(rightHandle);
//register pan event
rightHandleHammer.add(new Hammer.Pan({ threshold: 0, pointers: 0 }));
//register doubletap event
rightHandleHammer.add(new Hammer.Tap({ event: 'doubletap', taps: 2 }));

//right handle pan event
rightHandleHammer.on('panstart', function (e) {
    startWidth = iFrameContainer.clientWidth;
    notifyPanResizeStart(Handle.Right);
});
rightHandleHammer.on('panend', function (e) {
    notifyPanResizeEnd(Handle.Right);
});
rightHandleHammer.on('panmove', function (e) {
    if (!isResizeEnabled) {
        notifyResizeAttemptWhenDisabled();
        return;
    }
    setFrameWidthInternal(startWidth + e.deltaX, false);
})
//right hande doubletap event
rightHandleHammer.on("doubletap", function (e) {
    notifyDoubleClickHandle(Handle.Right)
});


//bottomHandle
var bottomHandleHammer = new Hammer.Manager(bottomHandle);
//register pan event
bottomHandleHammer.add(new Hammer.Pan({ direction: Hammer.DIRECTION_VERTICAL }));
//register doubletap event
bottomHandleHammer.add(new Hammer.Tap({ event: 'doubletap', taps: 2 }));
// bottomHandleHammer.get('pan').recognizeWith('doubletap');


bottomHandleHammer.on('panstart', function (e) {
    startHeight = iFrameContainer.clientHeight;
    notifyPanResizeStart(Handle.Bottom);
});
bottomHandleHammer.on('panend', function (e) {
    notifyPanResizeEnd(Handle.Bottom);
});
bottomHandleHammer.on('panmove', function (e) {
    if (!isResizeEnabled) {
        notifyResizeAttemptWhenDisabled();
        return;
    }
    setFrameHeightInternal(startHeight + e.deltaY, false);
})
bottomHandleHammer.on("doubletap", function (e) {
    notifyDoubleClickHandle(Handle.Bottom)
});




//cornerHandle
var cornerHandleHammer = new Hammer.Manager(cornerHandle);
//register pan event
cornerHandleHammer.add(new Hammer.Pan({ threshold: 0, pointers: 0 }));
//register doubletap event
cornerHandleHammer.add(new Hammer.Tap({ event: 'doubletap', taps: 2 }));

cornerHandleHammer.on('panstart', function (e) {    
    startHeight = iFrameContainer.clientHeight;
    startWidth = iFrameContainer.clientWidth;
    notifyPanResizeStart(Handle.Corner)
});
cornerHandleHammer.on('panend', function (e) {
    notifyPanResizeEnd(Handle.Corner)
});
cornerHandleHammer.on('panmove', function (e) {
    if (!isResizeEnabled) {
        notifyResizeAttemptWhenDisabled();
        return;
    }
    setFrameHeightInternal(startHeight + e.deltaY, false);
    setFrameWidthInternal(startWidth + e.deltaX, false);
})
cornerHandleHammer.on("doubletap", function (e) {
    notifyDoubleClickHandle(Handle.Corner)
});


function onResize(entries) {
    var iframeRect = entries[0].contentRect;

    var width = Math.round(iframeRect.width).toString();
    var height = Math.round(iframeRect.height).toString();

    notifyDimensionsChanges(width, height);
}

function setFrameUrl(url) {
    iFrame.src = url;
}

function setWindowZoom(zoom) {
    document.body.style.zoom = zoom + "%";
}

function setFrameWidth(width) {
    setFrameWidthInternal(width, true);
}

/**
 * Internal method. Only to be called here
 */
function setFrameWidthInternal(width, shouldNotify) {
    var width_float = parseFloat(width);

    if (width_float < MIN_FRAME_WIDTH) {
        setFrameWidthInternal(MIN_FRAME_WIDTH, shouldNotify);
        return;
    }
    
    var oldWidth = iFrameContainer.clientWidth
    if (width_float == oldWidth) {
        return
    }

    iFrameContainer.style.width = width + "px";
    if (shouldNotify) {
        notifyOnSetWidth(width_float, oldWidth);   
    }
}

function setFrameHeight(height) {
    setFrameHeightInternal(height, true);
}

/**
 * Internal method. Only to be called here
 */
function setFrameHeightInternal(height, shouldNotify) {
    var height_float = parseInt(height);

    if (height_float < MIN_FRAME_HEIGHT) {
        setFrameHeightInternal(MIN_FRAME_HEIGHT, shouldNotify);
        return;
    }

    var oldHeight = iFrameContainer.clientHeight
    if (height_float == oldHeight) {
        return
    }

    iFrameContainer.style.height = `${height}px`;
    if (shouldNotify) {
        notifyOnSetHeight(height_float, oldHeight);
    }
}


/**
 * Sets the emulator width and height and notifies Android of the change 
 * @param {number} width width value
 * @param {number} height height value
 */
function setFrameDimens(width, height) {
    setFrameWidth(width);
    setFrameHeight(height);
}

function setIsResizeEnabled(isEnabled) {
    isResizeEnabled = isEnabled;
}


function setFrameHeightToMatchViewportHeight() {
    var windowWidth = window.innerWidth
    if (windowWidth > 768) {
        //tablet device or grater

        iFrameContainer.style.height = (window.innerHeight * 0.9) + "px";
    } else{
        //a mobile device
        
        iFrameContainer.style.height = (window.innerHeight * 0.88) + "px";
    }
}

function setFrameWidthToMatchViewportWidth() {
    var windowWidth = window.innerWidth
    if (windowWidth > 768) {
        //tablet device or greater

        iFrameContainer.style.width = (windowWidth * 0.88) + "px";
    } else{
        //a mobile device
        
        iFrameContainer.style.width = (windowWidth * 0.80) + "px";
    }
}

function showEmulatorHandles() {
    bottomHandle.style.visibility = "visible"
    rightHandle.style.visibility = "visible"
    cornerHandle.style.visibility = "visible"
}

function hideEmulatorHandles() {
    bottomHandle.style.visibility = "hidden"
    rightHandle.style.visibility = "hidden"
    cornerHandle.style.visibility = "hidden"
}

function hideEmulatorCutout() {
    setElementStyle(emulatorCutout, {
        "visibility" : "hidden",
    })
}


function showEmulatorCutout(size, cutoutOrientation){
    var vert = cutoutOrientation == CutoutOrientation.VERTICALLY_CENTERED
    setElementStyle(emulatorCutout, {
        "width" : (vert ? size + "px" : "100%"),
        "height" : (vert ? "100%" : size + "px"),
        "visibility" : "visible"
    })
}

/**
 * Sets the style of and element with a given id
 * @param {String} objId the element id
 * @param {*} propertyObject 
 */
function setStyle(objId, propertyObject){
    var elem = document.getElementById(objId);
    setElementStyle(elem, propertyObject)
}

/**
 * Sets the style of and element
 * @param {Element} objId the element
 * @param {*} propertyObject 
 */
function setElementStyle(element, propertyObject) {
    for (var property in propertyObject){
        element.style[property] = propertyObject[property];
    }
}


function notifyDimensionsChanges(width, height) {
    handleWidthChangedInternal(width)
    try {
        Android.onDimensionsChanged(parseInt(width), parseInt(height));
    } catch (error) {
        var windowWidth = window.innerWidth
        var windowHeight = windowWidth.innerHeight

        Android.onDimensionsChanged(parseInt(windowWidth * 0.8), parseInt(windowHeight * 0.88));
    }
}

function notifyPanResizeStart(handle) {
    Android.onPanResizeStart(handle)
}

function notifyPanResizeEnd(handle){
    Android.onPanResizeEnd(handle)
}

function notifyDoubleClickHandle(handle){
    Android.onDoubleClickHandle(handle)
}


function notifyOnSetWidth(width, oldWidth) {
    Android.onSetWidth(width, oldWidth)
}

function notifyOnSetHeight(height, oldHeight) {
    Android.onSetHeight(height, oldHeight)
}

function notifyResizeAttemptWhenDisabled() {
    Android.onAttemptResizeWhenDisabled();
}

function initMutationObserver() {
    var MutationObserver = window.MutationObserver || window.WebKitMutationObserver || window.MozMutationObserver;

    var list = document.getElementById("iframe_container");

    var config = {
        attributes: true,
        childList: false,
        characterData: false,
        subtree: false,
    };
    var observer = new MutationObserver(function (mutations) {
        mutations.forEach(function (mutation) {
            if (mutation.type === "attributes") {

                notifyDimensionsChanges(parseInt(list.style.width), parseInt(list.style.height));
            }
        });
    });
    observer.observe(list, config);
}

function getWidthPointFromWidth(width) {
    if (width <= 350) {
        return WidthPoints._350_AND_LESS
    } else if (width <= 600) {
        return WidthPoints._350_TO_600
    } else if (width <= 950) {
        return WidthPoints._601_TO_950
    } else if (width <= 1150) {
        return WidthPoints._951_TO_1150
    } else {
        return WidthPoints._1151_AND_MORE
    }
}

function handleWidthChangedInternal(width) {
    var widthPoint = getWidthPointFromWidth(width)

    if (currentWidthPoint != widthPoint) {
        setHandleIconBasedOnWidth(currentWidthPoint, widthPoint)
        currentWidthPoint = widthPoint
    }
}

function getClassNameFromWidthPoint(widthPoint, hande) {
    switch (widthPoint) {
        case WidthPoints._350_AND_LESS:
            return hande == Handle.Corner ? "_350_and_less_corner": "_350_and_less"
        case WidthPoints._350_TO_600:
            return hande == Handle.Corner ? "_351_to_600_corner": "_351_to_600"
        case WidthPoints._601_TO_950:
            return hande == Handle.Corner ? "_601_to_950_corner": "_601_to_950"
            case WidthPoints._951_TO_1150:
                return hande == Handle.Corner ? "_951_to_1150_corner": "_951_to_1150"
        default:
            return hande == Handle.Corner ? "_1151_and_more_corner": "_1151_and_more";
    }
}

function setHandleIconBasedOnWidth(oldtWidthPoint, newWidthPoint){
    if (oldtWidthPoint == newWidthPoint) {
        return
    }

    if (newWidthPoint == WidthPoints.NOT_SET) {
        newWidthPoint = getWidthPointFromWidth(parseInt(iFrameContainer.style.width))
    }

    bottomHandle.classList.remove(getClassNameFromWidthPoint(oldtWidthPoint, Handle.Bottom))
    rightHandle.classList.remove(getClassNameFromWidthPoint(oldtWidthPoint, Handle.Right))
    cornerHandle.classList.remove(getClassNameFromWidthPoint(oldtWidthPoint, Handle.Corner))

    bottomHandle.classList.add(getClassNameFromWidthPoint(newWidthPoint, Handle.Bottom))
    rightHandle.classList.add(getClassNameFromWidthPoint(newWidthPoint, Handle.Right))
    cornerHandle.classList.add(getClassNameFromWidthPoint(newWidthPoint, Handle.Corner))
}